# coding=utf-8
import random
from collections import OrderedDict

from apolloCommon import mysqlPool

from soldierLotteryScripts import consts


class MysqlManager(object):
    """
    MysqlManager主要用来管理数据库数据
    提供函数快捷获取玩家数据
    """

    def __init__(self, serviceSystem):
        self.mServiceSystem = serviceSystem
        print "初始化MySQL数据库连接"
        mysqlPool.InitDB(10)
        self.mPlayerCache = {}  # 玩家数据缓存
        self.mStateCache = OrderedDict()  # 彩票状态缓存
        self.mLotteryNumbers = {}  # 彩票号码缓存
        self.CacheLotteryStates()
        self.currentLottery = -1

    def GetUnfinishedLottery(self):
        for lottery, state in self.mStateCache.items():
            if state == 0:
                return lottery
        return -1

    def Destroy(self):
        mysqlPool.Finish()
        print "关闭MySQL连接"

    def FinishLottery(self, lottery):
        """
        强制某个彩票开奖
        :param lottery:int 彩票id
        :return:
        """
        if self.mStateCache[lottery] != 0:
            return False
        self.FinishSaveDoneCallback(lottery)

    def GetAllInvovedPlayers(self, lottery, players, numbers, super):
        def callback(result, players, numbers):
            allPlayers = []
            if result:
                for line in result:
                    player = int(line[0])
                    if player not in players and player != super:
                        allPlayers.append(player)
            self.mServiceSystem.SendNonRewards(allPlayers, numbers, super)
            sql = "UPDATE `{}` SET `finish` = 1 WHERE `id` = %s;".format(consts.LotteryStateTable)
            mysqlPool.AsyncExecuteWithOrderKey("finish_lottery_update", sql, (lottery,), None)
            sql = "DELETE FROM `{}` WHERE `lottery` = %s;".format(consts.LotteryPlayerTable)
            mysqlPool.AsyncExecuteWithOrderKey("finish_lottery_delete", sql, (lottery,), None)
            self.CreateNewLottery()  # 创建新一轮彩票

        sql = "SELECT DISTINCT `player` FROM `{}` WHERE `lottery` = %s;".format(consts.LotteryPlayerTable)
        mysqlPool.AsyncQueryWithOrderKey("finish_lottery_all", sql, (lottery,),
                                         lambda result: callback(result, players, numbers))

    def FinishSaveDoneCallback(self, lottery):
        def callback(result):
            players = []
            numbers = []
            if result:
                for line in result:
                    players.append(int(line[0]))
                    numbers.append(line[1])

            def query_super(result):
                superOwner = None
                if result:
                    superOwner = int(result[0][0])
                args = {"players": players, "numbers": numbers}
                self.mStateCache[lottery] = 1  # 标记开奖完成
                self.mServiceSystem.BroadcastEvent(consts.BeforeLotteryDrawEvent, args)
                self.mServiceSystem.SendRewards(args["players"], args["numbers"],
                                                superOwner)  # 将结果交给SendRewards函数进行发奖处理
                self.GetAllInvovedPlayers(lottery, args["players"], args["numbers"], superOwner)

            sql = "SELECT `player` FROM `{}` WHERE `lottery` = %s AND `number` = 6666;".format(
                consts.LotteryPlayerTable)
            mysqlPool.AsyncQueryWithOrderKey("query_super_award", sql, (lottery,), query_super)

        sql = "SELECT `player`,`number` FROM `{}` WHERE `lottery` = %s AND `number` <> 6666 ORDER BY RAND() LIMIT 5;".format(
            consts.LotteryPlayerTable)  # 使用SQL语句来随机选择5个中奖号码
        mysqlPool.AsyncQueryWithOrderKey("finish_lottery_rand", sql, (lottery,), callback)

    def AddPlayerLotteryNumber(self, player, lottery, number):
        """
        用来保存数据到数据库
        :param player:str 玩家uid
        :param lottery:int 彩票id
        :param number:int 彩票号码
        :return:
        """
        sql = "INSERT INTO `{}`(`player`,`lottery`,`number`) VALUES(%s,%s,%s);".format(consts.LotteryPlayerTable)
        mysqlPool.AsyncExecuteWithOrderKey("add_lottery_number_{}".format(player), sql, (player, lottery, number), None)

    def CreateNewLottery(self,cache = False):
        def callback(result):
            if result is None:
                print "创建彩票错误!"
            id = int(result)
            print "彩票创建成功,id{}".format(id)
            self.mStateCache[id] = 0
            if cache:
                self.CacheAllNumbers()

        sql = "INSERT INTO `{}` VALUES()".format(consts.LotteryStateTable)
        mysqlPool.AsyncInsertOneWithOrderKey("add_lottery", sql, (), callback)

    def CacheLotteryStates(self):
        sql = "SELECT `id` FROM `{}` WHERE `finish` = 0;".format(consts.LotteryStateTable)

        def callback(result):
            if result:
                id = result[0][0]
                print "找到彩票ID为{}".format(id)
                self.mStateCache[id] = 0
                self.CacheAllNumbers()
            else:
                print "未找到现成彩票，即将新建"
                self.CreateNewLottery(True)

        mysqlPool.AsyncQueryWithOrderKey("query_states", sql, (), callback)

    def CacheAllNumbers(self):
        sql = "SELECT `number` FROM `{}` WHERE `lottery` = %s;".format(consts.LotteryPlayerTable)
        for lotteryId, state in self.mStateCache.items():
            if state == 0:
                def callback(lotteryId, result):
                    numbers = []
                    if result:
                        for line in result:
                            numbers.append(int(line[0]))
                    print "彩票{}的号码为{}".format(lotteryId, numbers)
                    self.mLotteryNumbers[lotteryId] = numbers

                mysqlPool.AsyncQueryWithOrderKey("query_numbers_{}".format(lotteryId), sql, (lotteryId,),
                                                 lambda result: callback(lotteryId, result))

    def GetPlayerRandomNumber(self, uid, lottery):
        """
        让玩家随机获取一个数字
        :param uid:str 玩家UID
        :param lottery:int 彩票ID
        :return:int 数字，如果-1则失败
        """
        amount = self.GetPlayerNumbersAmount(uid, lottery)
        if amount >= 3:
            # 每个玩家最多3个号码
            print "{}超出上限，返回-1".format(uid)
            return -1
        allNumbers = range(1, 10001)
        existedNumbers = self.mLotteryNumbers.get(lottery, [])
        if len(existedNumbers) >= 10000:
            print "{}无可用号码，返回-1".format(lottery)
            return -1
        for number in existedNumbers:
            allNumbers.remove(number)
        index = random.randint(0, len(allNumbers)-1)
        randomNumber = allNumbers[index]
        self.AddPlayerLotteryNumber(uid, lottery, randomNumber)  # 修改实时保存到数据库
        cache = self.GetPlayerCache(uid)
        list = cache.get(lottery, [])
        list.append(randomNumber)  # 将抽取到的数字添加到玩家缓存
        existedNumbers.append(randomNumber)  # 将抽取到的数字添加到总缓存
        self.mLotteryNumbers[lottery] = existedNumbers
        cache[lottery] = list
        return randomNumber

    def GetPlayerCache(self, uid):
        """
        获取玩家缓存，没有缓存则从数据库获取，并返回{}
        :param uid:
        :return:
        """
        cache = self.mPlayerCache.get(uid)
        if cache is not None:
            return cache

        def callback(uid, result):
            data = {}
            if result:
                for line in result:
                    tmp = data.get(line[0], [])
                    tmp.append(line[1])
                    data[line[0]] = tmp
            print "缓存玩家{}的数据为{}".format(uid, data)
            self.mPlayerCache[uid] = data

        sql = "SELECT `lottery`,`number` FROM `{}` WHERE `player` = %s;".format(consts.LotteryPlayerTable)
        mysqlPool.AsyncQueryWithOrderKey("cache_player_{}".format(uid), sql, (uid,),
                                         lambda result: callback(uid, result))
        return None

    def RemovePlayerCache(self, uid):
        """
        移除玩家缓存，退出时触发
        :param uid: 玩家UID
        :return:None
        """
        cache = self.mPlayerCache.get(uid)
        if cache is None:
            # 因为正常获取过缓存数据的玩家，都存储过cache为{}，不可能是None。所以此处是None的话不处理
            return
        del self.mPlayerCache[uid]

    def GetPlayerNumbersAmount(self, uid, lottery):
        """
        获取玩家某个彩票的数量
        :param uid:str 玩家uid
        :param lottery:int 彩票id
        :return:int 数量
        """
        cache = self.GetPlayerCache(uid)
        if cache is None:
            print "错误，玩家{}未缓存，请检查!".format(uid)
            return -1
        return len(cache.get(lottery, []))

    def GetPlayerNumbers(self, uid, lottery):
        """
        获取玩家某个彩票的所有号码
        :param uid:str 玩家uid
        :param lottery:int 彩票id
        :return:list[int] 号码列表
        """
        cache = self.GetPlayerCache(uid)
        if cache is None:
            print "错误，玩家{}未缓存，请检查!".format(uid)
            return -1
        return cache.get(lottery, [])

    def ResetAllNumbers(self, lottery):
        """
        重置某个彩票的所有号码
        :param lottery:int 彩票ID
        :return:None
        """
        for uid in self.mPlayerCache.keys():
            self.ResetPlayerNumbers(uid, lottery, False)
        # 先清空本地缓存，再清理数据库
        sql = "DELETE FROM `{}` WHERE `lottery` = %s;".format(consts.LotteryPlayerTable)
        mysqlPool.AsyncExecuteWithOrderKey("reset_all_numbers", sql, (lottery,), None)
        return True

    def ResetPlayerNumbers(self, uid, lottery, database=True):
        """
        重置玩家的某个彩票
        :param uid:str 玩家UID
        :param lottery:int 彩票ID
        :return:bool 是否成功
        """
        cache = self.GetPlayerCache(uid)
        if database:
            sql = "DELETE FROM `{}` WHERE `player` = %s;".format(consts.LotteryPlayerTable)
            mysqlPool.AsyncExecuteWithOrderKey("reset_all_numbers_{}".format(uid), sql, (uid,), None)
        if cache is None:
            print "错误，玩家{}未缓存，请检查!".format(uid)
            return False
        if lottery in cache:
            del cache[lottery]
            return True
        return False
